{-# LANGUAGE PArr #-}
{-# OPTIONS -fvectorise #-}
module SpellCheckDPH where

import qualified Prelude
import Data.Array.Parallel.Prelude
import Data.Array.Parallel.Prelude.Word8
import qualified Data.Array.Parallel.Prelude.Int as I

type WordP = [:Word8:]
type DocumentP = [:WordP:]
type DictionaryP = [:TrieP:]
data TrieP = NodeP Word8 DictionaryP | LeafP Word8

isNode :: Word8 -> TrieP -> Bool
isNode w t = case t of
  (NodeP w' dic) -> w == w'
  _ -> False

isLeaf :: Word8 -> TrieP -> Bool
isLeaf w t = case t of
  (LeafP w') -> w == w'
  _ -> False

lookupTrieP :: DictionaryP -> I.Int -> WordP -> Bool
lookupTrieP dic i w =
    let len = lengthP w
        in if (I.<) ((I.+) i 1) len
              then let node = filterP (isNode (w!:i)) dic
                       in if (I.==) (lengthP node) 0
                             then False
                             else case node!:0 of
                                    (NodeP _ d) -> lookupTrieP d ((I.+) i 1) w
                                    _ -> False
              else (I.==) (lengthP (filterP (isLeaf (w!:i)) dic)) 1

spellCheckP :: DictionaryP -> DocumentP -> DocumentP
spellCheckP dic doc = filterP (\s -> not (lookupTrieP dic 0 s)) doc
