
module Trie where

data (Eq a) => Trie a = Node !a ![Trie a] | Leaf !a

lookupTrie :: (Eq a) => [Trie a] -> [a] -> Bool
lookupTrie _ [] = True
lookupTrie [] _ = False
lookupTrie (t:ts) [x] = case t of
    (Leaf y) -> if y == x then True else lookupTrie ts [x]
    _        -> lookupTrie ts [x]
lookupTrie (t:ts) (x:xs) = case t of
    (Node y ys) -> if x == y then lookupTrie ys xs
                             else lookupTrie ts (x:xs)
    _           -> lookupTrie ts (x:xs)

-- Insert a new word in StringTree
insertTrie :: (Eq a) => [a] -> [Trie a] -> [Trie a]

-- Empty
insertTrie [] t = t

-- Last character in string (insert as leaf)
insertTrie [x] [] = [Leaf x]
insertTrie [x] (y:ys) = case y of
    (Leaf z)   -> if z == x then (y:ys)
                else y:(insertTrie [x] ys)
    (Node _ _) -> y:(insertTrie [x] ys)

-- Other characters (insert as nodes)
insertTrie (x:xs) [] = [Node x (insertTrie xs [])]
insertTrie (x:xs) (y:ys) = case y of
    (Node z zs) -> if x == z then (Node z (insertTrie xs zs)):ys
                             else y:(insertTrie (x:xs) ys)
    _           -> y:(insertTrie (x:xs) ys)

-- List words in tree
trie2string :: [Trie Char] -> [Char] -> [Char]
trie2string [] w = ""
trie2string (t:ts) w = case t of
    (Leaf c) -> w ++ [c] ++ "\n" ++ (trie2string ts w)
    (Node c ns) -> (trie2string ns (w++[c])) ++ (trie2string ts w)

